# encoding: utf-8
# module System.Windows.Forms.VisualStyles calls itself VisualStyles
# from System.Windows.Forms,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class BackgroundType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the source of a visual style element's background.

 

 enum BackgroundType,values: BorderFill (1),ImageFile (0),None (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 BorderFill=None
 ImageFile=None
 None=None
 value__=None


class BooleanProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the Boolean properties of a visual style element.

 

 enum BooleanProperty,values: AlwaysShowSizingBar (2208),AutoSize (2202),BackgroundFill (2205),BorderOnly (2203),Composited (2204),GlyphOnly (2207),GlyphTransparent (2206),IntegralSizing (2211),MirrorImage (2209),SourceGrow (2212),SourceShrink (2213),Transparent (2201),UniformSizing (2210)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AlwaysShowSizingBar=None
 AutoSize=None
 BackgroundFill=None
 BorderOnly=None
 Composited=None
 GlyphOnly=None
 GlyphTransparent=None
 IntegralSizing=None
 MirrorImage=None
 SourceGrow=None
 SourceShrink=None
 Transparent=None
 UniformSizing=None
 value__=None


class BorderType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the border type of a visual style element with a filled-border background.

 

 enum BorderType,values: Ellipse (2),Rectangle (0),RoundedRectangle (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Ellipse=None
 Rectangle=None
 RoundedRectangle=None
 value__=None


class CheckBoxState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a check box that is drawn with visual styles.

 

 enum CheckBoxState,values: CheckedDisabled (8),CheckedHot (6),CheckedNormal (5),CheckedPressed (7),MixedDisabled (12),MixedHot (10),MixedNormal (9),MixedPressed (11),UncheckedDisabled (4),UncheckedHot (2),UncheckedNormal (1),UncheckedPressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 CheckedDisabled=None
 CheckedHot=None
 CheckedNormal=None
 CheckedPressed=None
 MixedDisabled=None
 MixedHot=None
 MixedNormal=None
 MixedPressed=None
 UncheckedDisabled=None
 UncheckedHot=None
 UncheckedNormal=None
 UncheckedPressed=None
 value__=None


class ColorProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the color properties of a visual style element.

 

 enum ColorProperty,values: AccentColorHint (3823),BorderColor (3801),BorderColorHint (3822),EdgeDarkShadowColor (3807),EdgeFillColor (3808),EdgeHighlightColor (3805),EdgeLightColor (3804),EdgeShadowColor (3806),FillColor (3802),FillColorHint (3821),GlowColor (3816),GlyphTextColor (3819),GlyphTransparentColor (3820),GradientColor1 (3810),GradientColor2 (3811),GradientColor3 (3812),GradientColor4 (3813),GradientColor5 (3814),ShadowColor (3815),TextBorderColor (3817),TextColor (3803),TextShadowColor (3818),TransparentColor (3809)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AccentColorHint=None
 BorderColor=None
 BorderColorHint=None
 EdgeDarkShadowColor=None
 EdgeFillColor=None
 EdgeHighlightColor=None
 EdgeLightColor=None
 EdgeShadowColor=None
 FillColor=None
 FillColorHint=None
 GlowColor=None
 GlyphTextColor=None
 GlyphTransparentColor=None
 GradientColor1=None
 GradientColor2=None
 GradientColor3=None
 GradientColor4=None
 GradientColor5=None
 ShadowColor=None
 TextBorderColor=None
 TextColor=None
 TextShadowColor=None
 TransparentColor=None
 value__=None


class ComboBoxState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a combo box that is drawn with visual styles.

 

 enum ComboBoxState,values: Disabled (4),Hot (2),Normal (1),Pressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Disabled=None
 Hot=None
 Normal=None
 Pressed=None
 value__=None


class ContentAlignment(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies how text is aligned in a window caption.

 

 enum ContentAlignment,values: Center (1),Left (0),Right (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Center=None
 Left=None
 Right=None
 value__=None


class EdgeEffects(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual effects that can be applied to the edges of a visual style element.

 

 enum (flags) EdgeEffects,values: FillInterior (2048),Flat (4096),Mono (32768),None (0),Soft (16384)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 FillInterior=None
 Flat=None
 Mono=None
 None=None
 Soft=None
 value__=None


class Edges(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies which edges of a visual style element to draw.

 

 enum (flags) Edges,values: Bottom (8),Diagonal (16),Left (1),Right (4),Top (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Bottom=None
 Diagonal=None
 Left=None
 Right=None
 Top=None
 value__=None


class EdgeStyle(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the styles that can be applied to the edges of a visual style element.

 

 enum EdgeStyle,values: Bump (9),Etched (6),Raised (5),Sunken (10)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Bump=None
 Etched=None
 Raised=None
 Sunken=None
 value__=None


class EnumProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the enumerated type properties of a visual style element.

 

 enum EnumProperty,values: BackgroundType (4001),BorderType (4002),ContentAlignment (4006),FillType (4003),GlyphFontSizingType (4014),GlyphType (4012),HorizontalAlignment (4005),IconEffect (4009),ImageLayout (4011),ImageSelectType (4013),OffsetType (4008),SizingType (4004),TextShadowType (4010),TrueSizeScalingType (4015),VerticalAlignment (4007)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 BackgroundType=None
 BorderType=None
 ContentAlignment=None
 FillType=None
 GlyphFontSizingType=None
 GlyphType=None
 HorizontalAlignment=None
 IconEffect=None
 ImageLayout=None
 ImageSelectType=None
 OffsetType=None
 SizingType=None
 TextShadowType=None
 TrueSizeScalingType=None
 value__=None
 VerticalAlignment=None


class FilenameProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the names of the image files that are used to draw a visual style element.

 

 enum FilenameProperty,values: GlyphImageFile (3008),ImageFile (3001),ImageFile1 (3002),ImageFile2 (3003),ImageFile3 (3004),ImageFile4 (3005),ImageFile5 (3006),StockImageFile (3007)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 GlyphImageFile=None
 ImageFile=None
 ImageFile1=None
 ImageFile2=None
 ImageFile3=None
 ImageFile4=None
 ImageFile5=None
 StockImageFile=None
 value__=None


class FillType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the interior of visual style elements with a filled border background.

 

 enum FillType,values: HorizontalGradient (2),RadialGradient (3),Solid (0),TileImage (4),VerticalGradient (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 HorizontalGradient=None
 RadialGradient=None
 Solid=None
 TileImage=None
 value__=None
 VerticalGradient=None


class FontProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the font properties of a visual style element.

 

 enum FontProperty,values: GlyphFont (2601)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 GlyphFont=None
 value__=None


class GlyphFontSizingType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies when the visual style selects a different glyph font size.

 

 enum GlyphFontSizingType,values: Dpi (2),None (0),Size (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Dpi=None
 None=None
 Size=None
 value__=None


class GlyphType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the type of glyph for elements with a bitmap background.

 

 enum GlyphType,values: FontGlyph (2),ImageGlyph (1),None (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 FontGlyph=None
 ImageGlyph=None
 None=None
 value__=None


class GroupBoxState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a group box that is drawn with visual styles.

 

 enum GroupBoxState,values: Disabled (2),Normal (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Disabled=None
 Normal=None
 value__=None


class HitTestCode(Enum,IComparable,IFormattable,IConvertible):
 """
 Describes the location of a point in the background specified by a visual style.

 

 enum HitTestCode,values: Bottom (15),BottomLeft (16),BottomRight (17),Client (1),Left (10),Nowhere (0),Right (11),Top (12),TopLeft (13),TopRight (14)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Bottom=None
 BottomLeft=None
 BottomRight=None
 Client=None
 Left=None
 Nowhere=None
 Right=None
 Top=None
 TopLeft=None
 TopRight=None
 value__=None


class HitTestOptions(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the options that can be used when performing a hit test on the background specified by a visual style.

 

 enum (flags) HitTestOptions,values: BackgroundSegment (0),Caption (4),FixedBorder (2),ResizingBorder (240),ResizingBorderBottom (128),ResizingBorderLeft (16),ResizingBorderRight (64),ResizingBorderTop (32),SizingTemplate (256),SystemSizingMargins (512)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 BackgroundSegment=None
 Caption=None
 FixedBorder=None
 ResizingBorder=None
 ResizingBorderBottom=None
 ResizingBorderLeft=None
 ResizingBorderRight=None
 ResizingBorderTop=None
 SizingTemplate=None
 SystemSizingMargins=None
 value__=None


class HorizontalAlign(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the horizontal alignment for visual style elements with a fixed size.

 

 enum HorizontalAlign,values: Center (1),Left (0),Right (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Center=None
 Left=None
 Right=None
 value__=None


class IconEffect(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual effect that the visual style will apply to an icon.

 

 enum IconEffect,values: Alpha (4),Glow (1),None (0),Pulse (3),Shadow (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Alpha=None
 Glow=None
 None=None
 Pulse=None
 Shadow=None
 value__=None


class ImageOrientation(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies how multiple images are arranged in a single image file.

 

 enum ImageOrientation,values: Horizontal (1),Vertical (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Horizontal=None
 value__=None
 Vertical=None


class ImageSelectType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies when the visual style selects a different multiple-image file to draw an element.

 

 enum ImageSelectType,values: Dpi (2),None (0),Size (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Dpi=None
 None=None
 Size=None
 value__=None


class IntegerProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the integer properties of a visual style element.

 

 enum IntegerProperty,values: AlphaLevel (2402),AlphaThreshold (2415),BorderSize (2403),GlyphIndex (2418),GradientRatio1 (2406),GradientRatio2 (2407),GradientRatio3 (2408),GradientRatio4 (2409),GradientRatio5 (2410),Height (2417),ImageCount (2401),MinDpi1 (2420),MinDpi2 (2421),MinDpi3 (2422),MinDpi4 (2423),MinDpi5 (2424),ProgressChunkSize (2411),ProgressSpaceSize (2412),RoundCornerHeight (2405),RoundCornerWidth (2404),Saturation (2413),TextBorderSize (2414),TrueSizeStretchMark (2419),Width (2416)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AlphaLevel=None
 AlphaThreshold=None
 BorderSize=None
 GlyphIndex=None
 GradientRatio1=None
 GradientRatio2=None
 GradientRatio3=None
 GradientRatio4=None
 GradientRatio5=None
 Height=None
 ImageCount=None
 MinDpi1=None
 MinDpi2=None
 MinDpi3=None
 MinDpi4=None
 MinDpi5=None
 ProgressChunkSize=None
 ProgressSpaceSize=None
 RoundCornerHeight=None
 RoundCornerWidth=None
 Saturation=None
 TextBorderSize=None
 TrueSizeStretchMark=None
 value__=None
 Width=None


class MarginProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the margin properties of a visual style element.

 

 enum MarginProperty,values: CaptionMargins (3603),ContentMargins (3602),SizingMargins (3601)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 CaptionMargins=None
 ContentMargins=None
 SizingMargins=None
 value__=None


class OffsetType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies where an offset is applied to a window element.

 

 enum OffsetType,values: AboveLastButton (12),BelowLastButton (13),BottomLeft (3),BottomMiddle (5),BottomRight (4),LeftOfCaption (8),LeftOfLastButton (10),MiddleLeft (6),MiddleRight (7),RightOfCaption (9),RightOfLastButton (11),TopLeft (0),TopMiddle (2),TopRight (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AboveLastButton=None
 BelowLastButton=None
 BottomLeft=None
 BottomMiddle=None
 BottomRight=None
 LeftOfCaption=None
 LeftOfLastButton=None
 MiddleLeft=None
 MiddleRight=None
 RightOfCaption=None
 RightOfLastButton=None
 TopLeft=None
 TopMiddle=None
 TopRight=None
 value__=None


class PointProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the point properties of a visual style element.

 

 enum PointProperty,values: MinSize (3403),MinSize1 (3404),MinSize2 (3405),MinSize3 (3406),MinSize4 (3407),MinSize5 (3408),Offset (3401),TextShadowOffset (3402)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 MinSize=None
 MinSize1=None
 MinSize2=None
 MinSize3=None
 MinSize4=None
 MinSize5=None
 Offset=None
 TextShadowOffset=None
 value__=None


class PushButtonState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a button that is drawn with visual styles.

 

 enum PushButtonState,values: Default (5),Disabled (4),Hot (2),Normal (1),Pressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Default=None
 Disabled=None
 Hot=None
 Normal=None
 Pressed=None
 value__=None


class RadioButtonState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of an option button (also known as a radio button) that is drawn with visual styles.

 

 enum RadioButtonState,values: CheckedDisabled (8),CheckedHot (6),CheckedNormal (5),CheckedPressed (7),UncheckedDisabled (4),UncheckedHot (2),UncheckedNormal (1),UncheckedPressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 CheckedDisabled=None
 CheckedHot=None
 CheckedNormal=None
 CheckedPressed=None
 UncheckedDisabled=None
 UncheckedHot=None
 UncheckedNormal=None
 UncheckedPressed=None
 value__=None


class ScrollBarArrowButtonState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a scroll arrow that is drawn with visual styles.

 

 enum ScrollBarArrowButtonState,values: DownDisabled (8),DownHot (6),DownNormal (5),DownPressed (7),LeftDisabled (12),LeftHot (10),LeftNormal (9),LeftPressed (11),RightDisabled (16),RightHot (14),RightNormal (13),RightPressed (15),UpDisabled (4),UpHot (2),UpNormal (1),UpPressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 DownDisabled=None
 DownHot=None
 DownNormal=None
 DownPressed=None
 LeftDisabled=None
 LeftHot=None
 LeftNormal=None
 LeftPressed=None
 RightDisabled=None
 RightHot=None
 RightNormal=None
 RightPressed=None
 UpDisabled=None
 UpHot=None
 UpNormal=None
 UpPressed=None
 value__=None


class ScrollBarSizeBoxState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a scroll bar sizing handle that is drawn with visual styles.

 

 enum ScrollBarSizeBoxState,values: LeftAlign (2),RightAlign (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 LeftAlign=None
 RightAlign=None
 value__=None


class ScrollBarState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a scroll bar that is drawn with visual styles.

 

 enum ScrollBarState,values: Disabled (4),Hot (2),Normal (1),Pressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Disabled=None
 Hot=None
 Normal=None
 Pressed=None
 value__=None


class SizingType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies how elements with a bitmap background will adjust to fill a bounds.

 

 enum SizingType,values: FixedSize (0),Stretch (1),Tile (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 FixedSize=None
 Stretch=None
 Tile=None
 value__=None


class StringProperty(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the string properties of a visual style element.

 

 enum StringProperty,values: Text (3201)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Text=None
 value__=None


class TabItemState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a tab item that is drawn with visual styles.

 

 enum TabItemState,values: Disabled (4),Hot (2),Normal (1),Selected (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Disabled=None
 Hot=None
 Normal=None
 Selected=None
 value__=None


class TextBoxState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a text box that is drawn with visual styles.

 

 enum TextBoxState,values: Assist (7),Disabled (4),Hot (2),Normal (1),Readonly (6),Selected (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Assist=None
 Disabled=None
 Hot=None
 Normal=None
 Readonly=None
 Selected=None
 value__=None


class TextMetrics(object):
 """ Provides basic information about the font specified by a visual style for a particular element. """
 Ascent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the ascent of characters in the font.



Get: Ascent(self: TextMetrics) -> int



Set: Ascent(self: TextMetrics)=value

"""

 AverageCharWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the average width of characters in the font.



Get: AverageCharWidth(self: TextMetrics) -> int



Set: AverageCharWidth(self: TextMetrics)=value

"""

 BreakChar=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the character used to define word breaks for text justification.



Get: BreakChar(self: TextMetrics) -> Char



Set: BreakChar(self: TextMetrics)=value

"""

 CharSet=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the character set of the font.



Get: CharSet(self: TextMetrics) -> TextMetricsCharacterSet



Set: CharSet(self: TextMetrics)=value

"""

 DefaultChar=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the character to be substituted for characters not in the font.



Get: DefaultChar(self: TextMetrics) -> Char



Set: DefaultChar(self: TextMetrics)=value

"""

 Descent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the descent of characters in the font.



Get: Descent(self: TextMetrics) -> int



Set: Descent(self: TextMetrics)=value

"""

 DigitizedAspectX=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the horizontal aspect of the device for which the font was designed.



Get: DigitizedAspectX(self: TextMetrics) -> int



Set: DigitizedAspectX(self: TextMetrics)=value

"""

 DigitizedAspectY=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the vertical aspect of the device for which the font was designed.



Get: DigitizedAspectY(self: TextMetrics) -> int



Set: DigitizedAspectY(self: TextMetrics)=value

"""

 ExternalLeading=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the amount of extra leading that the application adds between rows.



Get: ExternalLeading(self: TextMetrics) -> int



Set: ExternalLeading(self: TextMetrics)=value

"""

 FirstChar=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the first character defined in the font.



Get: FirstChar(self: TextMetrics) -> Char



Set: FirstChar(self: TextMetrics)=value

"""

 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the height of characters in the font.



Get: Height(self: TextMetrics) -> int



Set: Height(self: TextMetrics)=value

"""

 InternalLeading=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the amount of leading inside the bounds set by the System.Windows.Forms.VisualStyles.TextMetrics.Height property.



Get: InternalLeading(self: TextMetrics) -> int



Set: InternalLeading(self: TextMetrics)=value

"""

 Italic=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the font is italic.



Get: Italic(self: TextMetrics) -> bool



Set: Italic(self: TextMetrics)=value

"""

 LastChar=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the last character defined in the font.



Get: LastChar(self: TextMetrics) -> Char



Set: LastChar(self: TextMetrics)=value

"""

 MaxCharWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the width of the widest character in the font.



Get: MaxCharWidth(self: TextMetrics) -> int



Set: MaxCharWidth(self: TextMetrics)=value

"""

 Overhang=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the extra width per string that may be added to some synthesized fonts.



Get: Overhang(self: TextMetrics) -> int



Set: Overhang(self: TextMetrics)=value

"""

 PitchAndFamily=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets information about the pitch,technology,and family of a physical font.



Get: PitchAndFamily(self: TextMetrics) -> TextMetricsPitchAndFamilyValues



Set: PitchAndFamily(self: TextMetrics)=value

"""

 StruckOut=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the font specifies a horizontal line through the characters.



Get: StruckOut(self: TextMetrics) -> bool



Set: StruckOut(self: TextMetrics)=value

"""

 Underlined=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the font is underlined.



Get: Underlined(self: TextMetrics) -> bool



Set: Underlined(self: TextMetrics)=value

"""

 Weight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the weight of the font.



Get: Weight(self: TextMetrics) -> int



Set: Weight(self: TextMetrics)=value

"""



class TextMetricsCharacterSet(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the character set of the font specified by a visual style for a particular element.

 

 enum TextMetricsCharacterSet,values: Ansi (0),Arabic (178),Baltic (186),ChineseBig5 (136),Default (1),EastEurope (238),Gb2312 (134),Greek (161),Hangul (129),Hebrew (177),Johab (130),Mac (77),Oem (255),Russian (204),ShiftJis (128),Symbol (2),Thai (222),Turkish (162),Vietnamese (163)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Ansi=None
 Arabic=None
 Baltic=None
 ChineseBig5=None
 Default=None
 EastEurope=None
 Gb2312=None
 Greek=None
 Hangul=None
 Hebrew=None
 Johab=None
 Mac=None
 Oem=None
 Russian=None
 ShiftJis=None
 Symbol=None
 Thai=None
 Turkish=None
 value__=None
 Vietnamese=None


class TextMetricsPitchAndFamilyValues(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies information about the pitch,technology,and family of the font specified by a visual style for a particular element.

 

 enum (flags) TextMetricsPitchAndFamilyValues,values: Device (8),FixedPitch (1),TrueType (4),Vector (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Device=None
 FixedPitch=None
 TrueType=None
 value__=None
 Vector=None


class TextShadowType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the type of shadow to add to text.

 

 enum TextShadowType,values: Continuous (2),None (0),Single (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Continuous=None
 None=None
 Single=None
 value__=None


class ThemeSizeType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the size of the visual style part to retrieve.

 

 enum ThemeSizeType,values: Draw (2),Minimum (0),True (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Draw=None
 Minimum=None
 True=None
 value__=None


class ToolBarState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a toolbar item that is drawn with visual styles.

 

 enum ToolBarState,values: Checked (5),Disabled (4),Hot (2),HotChecked (6),Normal (1),Pressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Checked=None
 Disabled=None
 Hot=None
 HotChecked=None
 Normal=None
 Pressed=None
 value__=None


class TrackBarThumbState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the visual state of a track bar slider (also known as a thumb) that is drawn with visual styles.

 

 enum TrackBarThumbState,values: Disabled (5),Hot (2),Normal (1),Pressed (3)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Disabled=None
 Hot=None
 Normal=None
 Pressed=None
 value__=None


class TrueSizeScalingType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the scaling type of a visual style element with a fixed size.

 

 enum TrueSizeScalingType,values: Dpi (2),None (0),Size (1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Dpi=None
 None=None
 Size=None
 value__=None


class VerticalAlignment(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the vertical alignment for visual style elements with a fixed size.

 

 enum VerticalAlignment,values: Bottom (2),Center (1),Top (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Bottom=None
 Center=None
 Top=None
 value__=None


class VisualStyleElement(object):
 """ Identifies a control or user interface (UI) element that is drawn with visual styles. """
 @staticmethod
 def CreateElement(className,part,state):
  """
  CreateElement(className: str,part: int,state: int) -> VisualStyleElement

  

   Creates a new visual style element from the specified class,part,and state values.

  

   className: A string that represents the class name of the visual style element to be created.

   part: A value that represents the part of the visual style element to be created.

   state: A value that represents the state of the visual style element to be created.

   Returns: A System.Windows.Forms.VisualStyles.VisualStyleElement with the 

    System.Windows.Forms.VisualStyles.VisualStyleElement.ClassName,

    System.Windows.Forms.VisualStyles.VisualStyleElement.Part,and 

    System.Windows.Forms.VisualStyles.VisualStyleElement.State properties initialized to the 

    className,part,and state parameters.
  """
  pass
 ClassName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the class name of the visual style element that this System.Windows.Forms.VisualStyles.VisualStyleElement represents.



Get: ClassName(self: VisualStyleElement) -> str



"""

 Part=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating the part of the visual style element that this System.Windows.Forms.VisualStyles.VisualStyleElement represents.



Get: Part(self: VisualStyleElement) -> int



"""

 State=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating the state of the visual style element that this System.Windows.Forms.VisualStyles.VisualStyleElement represents.



Get: State(self: VisualStyleElement) -> int



"""


 Button=None
 ComboBox=None
 ExplorerBar=None
 Header=None
 ListView=None
 Menu=None
 MenuBand=None
 Page=None
 ProgressBar=None
 Rebar=None
 ScrollBar=None
 Spin=None
 StartPanel=None
 Status=None
 Tab=None
 TaskBand=None
 Taskbar=None
 TaskbarClock=None
 TextBox=None
 ToolBar=None
 ToolTip=None
 TrackBar=None
 TrayNotify=None
 TreeView=None
 Window=None


class VisualStyleInformation(object):
 """ Provides information about the current visual style of the operating system. """
 Author='MSX'
 ColorScheme='NormalColor'
 Company=''
 ControlHighlightHot=None
 Copyright=''
 Description=''
 DisplayName='Aero style'
 IsEnabledByUser=True
 IsSupportedByOS=True
 MinimumColorDepth=0
 Size='NormalSize'
 SupportsFlatMenus=False
 TextControlBorder=None
 Url=''
 Version=''
 __all__=[]


class VisualStyleRenderer(object):
 """
 Provides methods for drawing and getting information about a System.Windows.Forms.VisualStyles.VisualStyleElement. This class cannot be inherited.

 

 VisualStyleRenderer(element: VisualStyleElement)

 VisualStyleRenderer(className: str,part: int,state: int)
 """
 def DrawBackground(self,dc,bounds,clipRectangle=None):
  """
  DrawBackground(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,clipRectangle: Rectangle)

   Draws the background image of the current visual style element within the specified bounding 

    rectangle and clipped to the specified clipping rectangle.

  

  

   dc: The System.Drawing.IDeviceContext used to draw the background image.

   bounds: A System.Drawing.Rectangle in which the background image is drawn.

   clipRectangle: A System.Drawing.Rectangle that defines a clipping rectangle for the drawing operation.

  DrawBackground(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle)

   Draws the background image of the current visual style element within the specified bounding 

    rectangle.

  

  

   dc: The System.Drawing.IDeviceContext used to draw the background image.

   bounds: A System.Drawing.Rectangle in which the background image is drawn.
  """
  pass
 def DrawEdge(self,dc,bounds,edges,style,effects):
  """
  DrawEdge(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,edges: Edges,style: EdgeStyle,effects: EdgeEffects) -> Rectangle

  

   Draws one or more edges of the specified bounding rectangle.

  

   dc: The System.Drawing.IDeviceContext used to draw the edges.

   bounds: The System.Drawing.Rectangle whose bounds define the edges to draw.

   edges: A bitwise combination of the System.Windows.Forms.VisualStyles.Edges values.

   style: A bitwise combination of the System.Windows.Forms.VisualStyles.EdgeStyle values.

   effects: A bitwise combination of the System.Windows.Forms.VisualStyles.EdgeEffects values.

   Returns: A System.Drawing.Rectangle that represents the interior of the bounds parameter,minus the edges 

    that were drawn.
  """
  pass
 def DrawImage(self,g,bounds,*__args):
  """
  DrawImage(self: VisualStyleRenderer,g: Graphics,bounds: Rectangle,imageList: ImageList,imageIndex: int)

   Draws the image from the specified System.Windows.Forms.ImageList within the specified bounds.

  

   g: The System.Drawing.Graphics used to draw the image.

   bounds: A System.Drawing.Rectangle in which the image is drawn.

   imageList: An System.Windows.Forms.ImageList that contains the System.Drawing.Image to draw.

   imageIndex: The index of the System.Drawing.Image within imageList to draw.

  DrawImage(self: VisualStyleRenderer,g: Graphics,bounds: Rectangle,image: Image)

   Draws the specified image within the specified bounds.

  

   g: The System.Drawing.Graphics used to draw the image.

   bounds: A System.Drawing.Rectangle in which the image is drawn.

   image: The System.Drawing.Image to draw.
  """
  pass
 def DrawParentBackground(self,dc,bounds,childControl):
  """
  DrawParentBackground(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,childControl: Control)

   Draws the background of a control's parent in the specified area.

  

   dc: The System.Drawing.IDeviceContext used to draw the background of the parent of childControl. 

    This object typically belongs to the child control.

  

   bounds: A System.Drawing.Rectangle in which to draw the parent control's background. This rectangle 

    

   childControl: The control whose parent's background will be drawn.
  """
  pass
 def DrawText(self,dc,bounds,textToDraw,drawDisabled=None,flags=None):
  """
  DrawText(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,textToDraw: str,drawDisabled: bool,flags: TextFormatFlags)

   Draws text in the specified bounding rectangle with the option of displaying disabled text and 

    applying other text formatting.

  

  

   dc: The System.Drawing.IDeviceContext used to draw the text.

   bounds: A System.Drawing.Rectangle in which to draw the text.

   textToDraw: The text to draw.

   drawDisabled: true to draw grayed-out text; otherwise,false.

   flags: A bitwise combination of the System.Windows.Forms.TextFormatFlags values.

  DrawText(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,textToDraw: str,drawDisabled: bool)

   Draws text in the specified bounds with the option of displaying disabled text.

  

   dc: The System.Drawing.IDeviceContext used to draw the text.

   bounds: A System.Drawing.Rectangle in which to draw the text.

   textToDraw: The text to draw.

   drawDisabled: true to draw grayed-out text; otherwise,false.

  DrawText(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,textToDraw: str)

   Draws text in the specified bounds using default formatting.

  

   dc: The System.Drawing.IDeviceContext used to draw the text.

   bounds: A System.Drawing.Rectangle in which to draw the text.

   textToDraw: The text to draw.
  """
  pass
 def GetBackgroundContentRectangle(self,dc,bounds):
  """
  GetBackgroundContentRectangle(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle) -> Rectangle

  

   Returns the content area for the background of the current visual style element.

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   bounds: A System.Drawing.Rectangle that contains the entire background area of the current visual style 

    element.

  

   Returns: A System.Drawing.Rectangle that contains the content area for the background of the current 

    visual style element.
  """
  pass
 def GetBackgroundExtent(self,dc,contentBounds):
  """
  GetBackgroundExtent(self: VisualStyleRenderer,dc: IDeviceContext,contentBounds: Rectangle) -> Rectangle

  

   Returns the entire background area for the current visual style element.

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   contentBounds: A System.Drawing.Rectangle that contains the content area of the current visual style element.

   Returns: A System.Drawing.Rectangle that contains the entire background area of the current visual style 

    element.
  """
  pass
 def GetBackgroundRegion(self,dc,bounds):
  """
  GetBackgroundRegion(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle) -> Region

  

   Returns the region for the background of the current visual style element.

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   bounds: A System.Drawing.Rectangle that contains the entire background area of the current visual style 

    element.

  

   Returns: The System.Drawing.Region that contains the background of the current visual style element.
  """
  pass
 def GetBoolean(self,prop):
  """
  GetBoolean(self: VisualStyleRenderer,prop: BooleanProperty) -> bool

  

   Returns the value of the specified Boolean property for the current visual style element.

  

   prop: One of the System.Windows.Forms.VisualStyles.BooleanProperty values that specifies which 

    property value to retrieve for the current visual style element.

  

   Returns: true if the property specified by the prop parameter is true for the current visual style 

    element; otherwise,false.
  """
  pass
 def GetColor(self,prop):
  """
  GetColor(self: VisualStyleRenderer,prop: ColorProperty) -> Color

  

   Returns the value of the specified color property for the current visual style element.

  

   prop: One of the System.Windows.Forms.VisualStyles.ColorProperty values that specifies which property 

    value to retrieve for the current visual style element.

  

   Returns: A System.Drawing.Color that contains the value of the property specified by the prop parameter 

    for the current visual style element.
  """
  pass
 def GetEnumValue(self,prop):
  """
  GetEnumValue(self: VisualStyleRenderer,prop: EnumProperty) -> int

  

   Returns the value of the specified enumerated type property for the current visual style element.

  

   prop: One of the System.Windows.Forms.VisualStyles.EnumProperty values that specifies which property 

    value to retrieve for the current visual style element.

  

   Returns: The integer value of the property specified by the prop parameter for the current visual style 

    element.
  """
  pass
 def GetFilename(self,prop):
  """
  GetFilename(self: VisualStyleRenderer,prop: FilenameProperty) -> str

  

   Returns the value of the specified file name property for the current visual style element.

  

   prop: One of the System.Windows.Forms.VisualStyles.FilenameProperty values that specifies which 

    property value to retrieve for the current visual style element.

  

   Returns: A System.String that contains the value of the property specified by the prop parameter for the 

    current visual style element.
  """
  pass
 def GetFont(self,dc,prop):
  """
  GetFont(self: VisualStyleRenderer,dc: IDeviceContext,prop: FontProperty) -> Font

  

   Returns the value of the specified font property for the current visual style element.

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   prop: One of the System.Windows.Forms.VisualStyles.FontProperty values that specifies which property 

    value to retrieve for the current visual style element.

  

   Returns: A System.Drawing.Font that contains the value of the property specified by the prop parameter 

    for the current visual style element.
  """
  pass
 def GetInteger(self,prop):
  """
  GetInteger(self: VisualStyleRenderer,prop: IntegerProperty) -> int

  

   Returns the value of the specified integer property for the current visual style element.

  

   prop: One of the System.Windows.Forms.VisualStyles.IntegerProperty values that specifies which 

    property value to retrieve for the current visual style element.

  

   Returns: The integer value of the property specified by the prop parameter for the current visual style 

    element.
  """
  pass
 def GetMargins(self,dc,prop):
  """
  GetMargins(self: VisualStyleRenderer,dc: IDeviceContext,prop: MarginProperty) -> Padding

  

   Returns the value of the specified margins property for the current visual style element.

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   prop: One of the System.Windows.Forms.VisualStyles.MarginProperty values that specifies which property 

    value to retrieve for the current visual style element.

  

   Returns: A System.Windows.Forms.Padding that contains the value of the property specified by the prop 

    parameter for the current visual style element.
  """
  pass
 def GetPartSize(self,dc,*__args):
  """
  GetPartSize(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,type: ThemeSizeType) -> Size

  

   Returns the value of the specified size property of the current visual style part using the 

    specified drawing bounds.

  

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   bounds: A System.Drawing.Rectangle that contains the area in which the part will be drawn.

   type: One of the System.Windows.Forms.VisualStyles.ThemeSizeType values that specifies which size 

    value to retrieve for the part.

  

   Returns: A System.Drawing.Size that contains the size specified by the type parameter for the current 

    visual style part.

  

  GetPartSize(self: VisualStyleRenderer,dc: IDeviceContext,type: ThemeSizeType) -> Size

  

   Returns the value of the specified size property of the current visual style part.

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   type: One of the System.Windows.Forms.VisualStyles.ThemeSizeType values that specifies which size 

    value to retrieve for the part.

  

   Returns: A System.Drawing.Size that contains the size specified by the type parameter for the current 

    visual style part.
  """
  pass
 def GetPoint(self,prop):
  """
  GetPoint(self: VisualStyleRenderer,prop: PointProperty) -> Point

  

   Returns the value of the specified point property for the current visual style element.

  

   prop: One of the System.Windows.Forms.VisualStyles.PointProperty values that specifies which property 

    value to retrieve for the current visual style element.

  

   Returns: A System.Drawing.Point that contains the value of the property specified by the prop parameter 

    for the current visual style element.
  """
  pass
 def GetString(self,prop):
  """
  GetString(self: VisualStyleRenderer,prop: StringProperty) -> str

  

   Returns the value of the specified string property for the current visual style element.

  

   prop: One of the System.Windows.Forms.VisualStyles.StringProperty values that specifies which property 

    value to retrieve for the current visual style element.

  

   Returns: A System.String that contains the value of the property specified by the prop parameter for the 

    current visual style element.
  """
  pass
 def GetTextExtent(self,dc,*__args):
  """
  GetTextExtent(self: VisualStyleRenderer,dc: IDeviceContext,bounds: Rectangle,textToDraw: str,flags: TextFormatFlags) -> Rectangle

  

   Returns the size and location of the specified string when drawn with the font of the current 

    visual style element within the specified initial bounding rectangle.

  

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   bounds: A System.Drawing.Rectangle used to control the flow and wrapping of the text.

   textToDraw: The string to measure.

   flags: A bitwise combination of the System.Windows.Forms.TextFormatFlags values.

   Returns: A System.Drawing.Rectangle that contains the area required to fit the rendered text.

  GetTextExtent(self: VisualStyleRenderer,dc: IDeviceContext,textToDraw: str,flags: TextFormatFlags) -> Rectangle

  

   Returns the size and location of the specified string when drawn with the font of the current 

    visual style element.

  

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   textToDraw: The string to measure.

   flags: A bitwise combination of the System.Windows.Forms.TextFormatFlags values.

   Returns: A System.Drawing.Rectangle that contains the area required to fit the rendered text.
  """
  pass
 def GetTextMetrics(self,dc):
  """
  GetTextMetrics(self: VisualStyleRenderer,dc: IDeviceContext) -> TextMetrics

  

   Retrieves information about the font specified by the current visual style element.

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   Returns: A System.Windows.Forms.VisualStyles.TextMetrics that provides information about the font 

    specified by the current visual style element.
  """
  pass
 def HitTestBackground(self,*__args):
  """
  HitTestBackground(self: VisualStyleRenderer,dc: IDeviceContext,backgroundRectangle: Rectangle,hRgn: IntPtr,pt: Point,options: HitTestOptions) -> HitTestCode

  

   Returns a hit test code indicating whether the point is contained in the background of the 

    current visual style element and within the specified region.

  

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   backgroundRectangle: A System.Drawing.Rectangle that contains the background of the current visual style element.

   hRgn: A Windows handle to a System.Drawing.Region that specifies the bounds of the hit test area 

    within the background.

  

   pt: The System.Drawing.Point to test.

   options: A bitwise combination of the System.Windows.Forms.VisualStyles.HitTestOptions values.

   Returns: A System.Windows.Forms.VisualStyles.HitTestCode that describes where pt is located in the 

    background of the current visual style element.

  

  HitTestBackground(self: VisualStyleRenderer,g: Graphics,backgroundRectangle: Rectangle,region: Region,pt: Point,options: HitTestOptions) -> HitTestCode

  

   Returns a hit test code indicating whether the point is contained in the background of the 

    current visual style element and within the specified bounds.

  

  

   g: The System.Drawing.Graphics this operation will use.

   backgroundRectangle: A System.Drawing.Rectangle that contains the background of the current visual style element.

   region: A System.Drawing.Region that specifies the bounds of the hit test area within the background.

   pt: The System.Drawing.Point to test.

   options: A bitwise combination of the System.Windows.Forms.VisualStyles.HitTestOptions values.

   Returns: A System.Windows.Forms.VisualStyles.HitTestCode that describes where pt is located in the 

    background of the current visual style element,if at all.

  

  HitTestBackground(self: VisualStyleRenderer,dc: IDeviceContext,backgroundRectangle: Rectangle,pt: Point,options: HitTestOptions) -> HitTestCode

  

   Returns a hit test code indicating whether a point is contained in the background of the current 

    visual style element.

  

  

   dc: The System.Drawing.IDeviceContext this operation will use.

   backgroundRectangle: A System.Drawing.Rectangle that contains the background of the current visual style element.

   pt: The System.Drawing.Point to test.

   options: A bitwise combination of the System.Windows.Forms.VisualStyles.HitTestOptions values.

   Returns: A System.Windows.Forms.VisualStyles.HitTestCode that describes where pt is located in the 

    background of the current visual style element.
  """
  pass
 def IsBackgroundPartiallyTransparent(self):
  """
  IsBackgroundPartiallyTransparent(self: VisualStyleRenderer) -> bool

  

   Indicates whether the background of the current visual style element has any semitransparent or 

    alpha-blended pieces.

  

   Returns: true if the background of the current visual style element has any semitransparent or 

    alpha-blended pieces; otherwise,false.
  """
  pass
 @staticmethod
 def IsElementDefined(element):
  """
  IsElementDefined(element: VisualStyleElement) -> bool

  

   Determines whether the specified visual style element is defined by the current visual style.

  

   element: A System.Windows.Forms.VisualStyles.VisualStyleElement whose class and part combination will be 

    verified.

  

   Returns: true if the combination of the System.Windows.Forms.VisualStyles.VisualStyleElement.ClassName 

    and System.Windows.Forms.VisualStyles.VisualStyleElement.Part properties of element are defined; 

    otherwise,false.
  """
  pass
 def SetParameters(self,*__args):
  """
  SetParameters(self: VisualStyleRenderer,className: str,part: int,state: int)

   Sets this System.Windows.Forms.VisualStyles.VisualStyleRenderer to the visual style element 

    represented by the specified class,part,and state values.

  

  

   className: The new value of the System.Windows.Forms.VisualStyles.VisualStyleRenderer.Class property.

   part: The new value of the System.Windows.Forms.VisualStyles.VisualStyleRenderer.Part property.

   state: The new value of the System.Windows.Forms.VisualStyles.VisualStyleRenderer.State property.

  SetParameters(self: VisualStyleRenderer,element: VisualStyleElement)

   Sets this System.Windows.Forms.VisualStyles.VisualStyleRenderer to the visual style element 

    represented by the specified System.Windows.Forms.VisualStyles.VisualStyleElement.

  

  

   element: A System.Windows.Forms.VisualStyles.VisualStyleElement that specifies the new values of the 

    System.Windows.Forms.VisualStyles.VisualStyleRenderer.Class,

    System.Windows.Forms.VisualStyles.VisualStyleRenderer.Part,and 

    System.Windows.Forms.VisualStyles.VisualStyleRenderer.State properties.
  """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,element: VisualStyleElement)

  __new__(cls: type,className: str,part: int,state: int)
  """
  pass
 Class=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the class name of the current visual style element.



Get: Class(self: VisualStyleRenderer) -> str



"""

 Handle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a unique identifier for the current class of visual style elements.



Get: Handle(self: VisualStyleRenderer) -> IntPtr



"""

 LastHResult=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the last error code returned by the native visual styles (UxTheme) API methods encapsulated by the System.Windows.Forms.VisualStyles.VisualStyleRenderer class.



Get: LastHResult(self: VisualStyleRenderer) -> int



"""

 Part=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the part of the current visual style element.



Get: Part(self: VisualStyleRenderer) -> int



"""

 State=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the state of the current visual style element.



Get: State(self: VisualStyleRenderer) -> int



"""


 IsSupported=True


class VisualStyleState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies how visual styles are applied to the current application.

 

 enum VisualStyleState,values: ClientAndNonClientAreasEnabled (3),ClientAreaEnabled (2),NonClientAreaEnabled (1),NoneEnabled (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 ClientAndNonClientAreasEnabled=None
 ClientAreaEnabled=None
 NonClientAreaEnabled=None
 NoneEnabled=None
 value__=None





class EventInstance(object):
 """
 Represents language-neutral information for an event log entry.

 

 EventInstance(instanceId: Int64,categoryId: int)

 EventInstance(instanceId: Int64,categoryId: int,entryType: EventLogEntryType)
 """
 @staticmethod
 def __new__(self,instanceId,categoryId,entryType=None):
  """
  __new__(cls: type,instanceId: Int64,categoryId: int)

  __new__(cls: type,instanceId: Int64,categoryId: int,entryType: EventLogEntryType)
  """
  pass
 CategoryId=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the resource identifier that specifies the application-defined category of the event entry.



Get: CategoryId(self: EventInstance) -> int



Set: CategoryId(self: EventInstance)=value

"""

 EntryType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the event type of the event log entry.



Get: EntryType(self: EventInstance) -> EventLogEntryType



Set: EntryType(self: EventInstance)=value

"""

 InstanceId=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the resource identifier that designates the message text of the event entry.



Get: InstanceId(self: EventInstance) -> Int64



Set: InstanceId(self: EventInstance)=value

"""





class ComponentCollection(ReadOnlyCollectionBase,ICollection,IEnumerable):
 """
 Provides a read-only container for a collection of System.ComponentModel.IComponent objects.

 

 ComponentCollection(components: Array[IComponent])
 """
 def CopyTo(self,array,index):
  """
  CopyTo(self: ComponentCollection,array: Array[IComponent],index: int)

   Copies the entire collection to an array,starting writing at the specified array index.

  

   array: An System.ComponentModel.IComponent array to copy the objects in the collection to.

   index: The index of the array at which copying to should begin.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 @staticmethod
 def __new__(self,components):
  """ __new__(cls: type,components: Array[IComponent]) """
  pass
 InnerList=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of elements contained in the System.Collections.ReadOnlyCollectionBase instance.



"""





class ObjectAuditRule(AuditRule):
 """ Represents a combination of a user's identity,an access mask,and audit conditions. An System.Security.AccessControl.ObjectAuditRule object also contains information about the type of object to which the rule applies,the type of child object that can inherit the rule,how the rule is inherited by child objects,and how that inheritance is propagated. """
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """ __new__(cls: type,identity: IdentityReference,accessMask: int,isInherited: bool,inheritanceFlags: InheritanceFlags,propagationFlags: PropagationFlags,objectType: Guid,inheritedObjectType: Guid,auditFlags: AuditFlags) """
  pass
 AccessMask=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the access mask for this rule.



"""

 InheritedObjectType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the type of child object that can inherit the stem.Security.AccessControl.ObjectAuditRule object.



Get: InheritedObjectType(self: ObjectAuditRule) -> Guid



"""

 ObjectFlags=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """System.Security.AccessControl.ObjectAuditRule.ObjectType and System.Security.AccessControl.ObjectAuditRule.InheritedObjectType properties of the stem.Security.AccessControl.ObjectAuditRule object contain valid values.



Get: ObjectFlags(self: ObjectAuditRule) -> ObjectAceFlags



"""

 ObjectType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the type of object to which the stem.Security.AccessControl.ObjectAuditRule applies.



Get: ObjectType(self: ObjectAuditRule) -> Guid



"""





class DataGridViewBindingCompleteEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.DataGridView.DataBindingComplete event.

 

 DataGridViewBindingCompleteEventArgs(listChangedType: ListChangedType)
 """
 @staticmethod
 def __new__(self,listChangedType):
  """ __new__(cls: type,listChangedType: ListChangedType) """
  pass
 ListChangedType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value specifying how the list changed.



Get: ListChangedType(self: DataGridViewBindingCompleteEventArgs) -> ListChangedType



"""





class CleanUpVirtualizedItemEventArgs(RoutedEventArgs):
 """
 Provides data for the System.Windows.Controls.VirtualizingStackPanel.CleanUpVirtualizedItem event.

 

 CleanUpVirtualizedItemEventArgs(value: object,element: UIElement)
 """
 @staticmethod
 def __new__(self,value,element):
  """ __new__(cls: type,value: object,element: UIElement) """
  pass
 Cancel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether this item should not be re-virtualized.



Get: Cancel(self: CleanUpVirtualizedItemEventArgs) -> bool



Set: Cancel(self: CleanUpVirtualizedItemEventArgs)=value

"""

 UIElement=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an instance of the visual element that represents the data value.



Get: UIElement(self: CleanUpVirtualizedItemEventArgs) -> UIElement



"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an System.Object that represents the original data value.



Get: Value(self: CleanUpVirtualizedItemEventArgs) -> object



"""





class DragEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.Control.DragDrop,System.Windows.Forms.Control.DragEnter,or System.Windows.Forms.Control.DragOver event.

 

 DragEventArgs(data: IDataObject,keyState: int,x: int,y: int,allowedEffect: DragDropEffects,effect: DragDropEffects)
 """
 @staticmethod
 def __new__(self,data,keyState,x,y,allowedEffect,effect):
  """ __new__(cls: type,data: IDataObject,keyState: int,x: int,y: int,allowedEffect: DragDropEffects,effect: DragDropEffects) """
  pass
 AllowedEffect=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets which drag-and-drop operations are allowed by the originator (or source) of the drag event.



Get: AllowedEffect(self: DragEventArgs) -> DragDropEffects



"""

 Data=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.IDataObject that contains the data associated with this event.



Get: Data(self: DragEventArgs) -> IDataObject



"""

 Effect=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the target drop effect in a drag-and-drop operation.



Get: Effect(self: DragEventArgs) -> DragDropEffects



Set: Effect(self: DragEventArgs)=value

"""

 KeyState=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current state of the SHIFT,CTRL,and ALT keys,as well as the state of the mouse buttons.



Get: KeyState(self: DragEventArgs) -> int



"""

 X=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the x-coordinate of the mouse pointer,in screen coordinates.



Get: X(self: DragEventArgs) -> int



"""

 Y=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the y-coordinate of the mouse pointer,in screen coordinates.



Get: Y(self: DragEventArgs) -> int



"""





class FontConverter(TypeConverter):
 """
 Converts System.Drawing.Font objects from one data type to another.

 

 FontConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: FontConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Determines whether this converter can convert an object in the specified source type to the 

    native type of the converter.

  

  

   context: A formatter context. This object can be used to get additional information about the environment 

    this converter is being called from. This may be null,so you should always check. Also,

    properties on the context object may also return null.

  

   sourceType: The type you want to convert from.

   Returns: This method returns true if this object can perform the conversion.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: FontConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object to the given destination 

    type using the context.

  

  

   context: An ITypeDescriptorContext object that provides a format context.

   destinationType: A System.Type object that represents the type you want to convert to.

   Returns: This method returns true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: FontConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the specified object to the native type of the converter.

  

   context: A formatter context. This object can be used to get additional information about the environment 

    this converter is being called from. This may be null,so you should always check. Also,

    properties on the context object may also return null.

  

   culture: A CultureInfo object that specifies the culture used to represent the font.

   value: The object to convert.

   Returns: The converted object.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: FontConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the specified object to another type.

  

   context: A formatter context. This object can be used to get additional information about the environment 

    this converter is being called from. This may be null,so you should always check. Also,

    properties on the context object may also return null.

  

   culture: A System.Globalization.CultureInfo object that specifies the culture used to represent the 

    object.

  

   value: The object to convert.

   destinationType: The data type to convert the object to.

   Returns: The converted object.
  """
  pass
 def CreateInstance(self,*__args):
  """
  CreateInstance(self: FontConverter,context: ITypeDescriptorContext,propertyValues: IDictionary) -> object

  

   Creates an object of this type by using a specified set of property values for the object.

  

   context: A type descriptor through which additional context can be provided.

   propertyValues: A dictionary of new property values. The dictionary contains a series of name-value pairs,one 

    for each property returned from the erload:System.Drawing.FontConverter.GetProperties method.

  

   Returns: The newly created object,or null if the object could not be created. The default implementation 

    returns 

    null.System.Drawing.FontConverter.CreateInstance(System.ComponentModel.ITypeDescriptorContext,Sys

    tem.Collections.IDictionary) useful for creating non-changeable objects that have changeable 

    properties.
  """
  pass
 def GetCreateInstanceSupported(self,context=None):
  """
  GetCreateInstanceSupported(self: FontConverter,context: ITypeDescriptorContext) -> bool

  

   Determines whether changing a value on this object should require a call to the 

    erload:System.Drawing.FontConverter.CreateInstance method to create a new value.

  

  

   context: A type descriptor through which additional context can be provided.

   Returns: This method returns true if the CreateInstance object should be called when a change is made to 

    one or more properties of this object; otherwise,false.
  """
  pass
 def GetProperties(self,*__args):
  """
  GetProperties(self: FontConverter,context: ITypeDescriptorContext,value: object,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   Retrieves the set of properties for this type. By default,a type does not have any properties 

    to return.

  

  

   context: A type descriptor through which additional context can be provided.

   value: The value of the object to get the properties for.

   attributes: An array of System.Attribute objects that describe the properties.

   Returns: The set of properties that should be exposed for this data type. If no properties should be 

    exposed,this may return null. The default implementation always returns null.An easy 

    implementation of this method can call the 

    erload:System.ComponentModel.TypeConverter.GetProperties method for the correct data type.
  """
  pass
 def GetPropertiesSupported(self,context=None):
  """
  GetPropertiesSupported(self: FontConverter,context: ITypeDescriptorContext) -> bool

  

   Determines whether this object supports properties. The default is false.

  

   context: A type descriptor through which additional context can be provided.

   Returns: This method returns true if the 

    System.Drawing.FontConverter.GetPropertiesSupported(System.ComponentModel.ITypeDescriptorContext)

     method should be called to find the properties of this object; otherwise,false.
  """
  pass
 FontNameConverter=None
 FontUnitConverter=None




class ControlStyles(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the style and behavior of a control.

 

 enum (flags) ControlStyles,values: AllPaintingInWmPaint (8192),CacheText (16384),ContainerControl (1),DoubleBuffer (65536),EnableNotifyMessage (32768),FixedHeight (64),FixedWidth (32),Opaque (4),OptimizedDoubleBuffer (131072),ResizeRedraw (16),Selectable (512),StandardClick (256),StandardDoubleClick (4096),SupportsTransparentBackColor (2048),UserMouse (1024),UserPaint (2),UseTextForAccessibility (262144)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AllPaintingInWmPaint=None
 CacheText=None
 ContainerControl=None
 DoubleBuffer=None
 EnableNotifyMessage=None
 FixedHeight=None
 FixedWidth=None
 Opaque=None
 OptimizedDoubleBuffer=None
 ResizeRedraw=None
 Selectable=None
 StandardClick=None
 StandardDoubleClick=None
 SupportsTransparentBackColor=None
 UserMouse=None
 UserPaint=None
 UseTextForAccessibility=None
 value__=None




class ToolTipService(object):
 """ Represents a service that provides properties and events to control the display and behavior of tooltips. """
 @staticmethod
 def AddToolTipClosingHandler(element,handler):
  """ AddToolTipClosingHandler(element: DependencyObject,handler: ToolTipEventHandler) """
  pass
 @staticmethod
 def AddToolTipOpeningHandler(element,handler):
  """ AddToolTipOpeningHandler(element: DependencyObject,handler: ToolTipEventHandler) """
  pass
 @staticmethod
 def GetBetweenShowDelay(element):
  """
  GetBetweenShowDelay(element: DependencyObject) -> int

  

   Gets the value of the System.Windows.Controls.ToolTipService.BetweenShowDelay�attached property 

    for an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.BetweenShowDelay property value.
  """
  pass
 @staticmethod
 def GetHasDropShadow(element):
  """
  GetHasDropShadow(element: DependencyObject) -> bool

  

   Gets the value of the System.Windows.Controls.ToolTipService.HasDropShadow�attached property for 

    an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.HasDropShadow property value.
  """
  pass
 @staticmethod
 def GetHorizontalOffset(element):
  """
  GetHorizontalOffset(element: DependencyObject) -> float

  

   Gets the value of the System.Windows.Controls.ToolTipService.HorizontalOffset�attached property 

    for an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.HorizontalOffset property value.
  """
  pass
 @staticmethod
 def GetInitialShowDelay(element):
  """
  GetInitialShowDelay(element: DependencyObject) -> int

  

   Gets the value of the System.Windows.Controls.ToolTipService.InitialShowDelay�attached property 

    for an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.InitialShowDelay property value.
  """
  pass
 @staticmethod
 def GetIsEnabled(element):
  """
  GetIsEnabled(element: DependencyObject) -> bool

  

   Gets the value of the System.Windows.Controls.ToolTipService.IsEnabled�attached property for an 

    object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.IsEnabled property value.
  """
  pass
 @staticmethod
 def GetIsOpen(element):
  """
  GetIsOpen(element: DependencyObject) -> bool

  

   Gets the value of the System.Windows.Controls.ToolTipService.IsOpen�attached property for an 

    object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.IsOpen property value.
  """
  pass
 @staticmethod
 def GetPlacement(element):
  """
  GetPlacement(element: DependencyObject) -> PlacementMode

  

   Gets the value of the System.Windows.Controls.ToolTipService.Placement�attached property for an 

    object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.Placement property value.
  """
  pass
 @staticmethod
 def GetPlacementRectangle(element):
  """
  GetPlacementRectangle(element: DependencyObject) -> Rect

  

   Gets the value of the System.Windows.Controls.ToolTipService.PlacementRectangle�attached 

    property for an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.PlacementRectangle property value.
  """
  pass
 @staticmethod
 def GetPlacementTarget(element):
  """
  GetPlacementTarget(element: DependencyObject) -> UIElement

  

   Gets the value of the System.Windows.Controls.ToolTipService.PlacementTarget�attached property 

    for an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.PlacementTarget property value.
  """
  pass
 @staticmethod
 def GetShowDuration(element):
  """
  GetShowDuration(element: DependencyObject) -> int

  

   Gets the value of the System.Windows.Controls.ToolTipService.ShowDuration�attached property for 

    an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.ShowDuration property value.
  """
  pass
 @staticmethod
 def GetShowOnDisabled(element):
  """
  GetShowOnDisabled(element: DependencyObject) -> bool

  

   Gets the value of the System.Windows.Controls.ToolTipService.ShowOnDisabled�attached property 

    for an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.ShowOnDisabled property value.
  """
  pass
 @staticmethod
 def GetToolTip(element):
  """
  GetToolTip(element: DependencyObject) -> object

  

   Gets the value of the System.Windows.Controls.ToolTipService.ToolTip�attached property for an 

    object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.ToolTip property value.
  """
  pass
 @staticmethod
 def GetVerticalOffset(element):
  """
  GetVerticalOffset(element: DependencyObject) -> float

  

   Gets the value of the System.Windows.Controls.ToolTipService.VerticalOffset�attached property 

    for an object.

  

  

   element: The object from which the property value is read.

   Returns: The object's System.Windows.Controls.ToolTipService.VerticalOffset property value.
  """
  pass
 @staticmethod
 def RemoveToolTipClosingHandler(element,handler):
  """ RemoveToolTipClosingHandler(element: DependencyObject,handler: ToolTipEventHandler) """
  pass
 @staticmethod
 def RemoveToolTipOpeningHandler(element,handler):
  """ RemoveToolTipOpeningHandler(element: DependencyObject,handler: ToolTipEventHandler) """
  pass
 @staticmethod
 def SetBetweenShowDelay(element,value):
  """
  SetBetweenShowDelay(element: DependencyObject,value: int)

   Sets the value of the System.Windows.Controls.ToolTipService.BetweenShowDelay�attached property 

    for an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetHasDropShadow(element,value):
  """
  SetHasDropShadow(element: DependencyObject,value: bool)

   Sets the value of the System.Windows.Controls.ToolTipService.HasDropShadow�attached property for 

    an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetHorizontalOffset(element,value):
  """
  SetHorizontalOffset(element: DependencyObject,value: float)

   Sets the value of the System.Windows.Controls.ToolTipService.HorizontalOffset�attached property 

    for an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetInitialShowDelay(element,value):
  """
  SetInitialShowDelay(element: DependencyObject,value: int)

   Sets the value of the System.Windows.Controls.ToolTipService.InitialShowDelay�attached property 

    for an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetIsEnabled(element,value):
  """
  SetIsEnabled(element: DependencyObject,value: bool)

   Sets the value of the System.Windows.Controls.ToolTipService.IsEnabled�attached property for an 

    object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetPlacement(element,value):
  """
  SetPlacement(element: DependencyObject,value: PlacementMode)

   Sets the value of the System.Windows.Controls.ToolTipService.Placement�attached property for an 

    object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetPlacementRectangle(element,value):
  """
  SetPlacementRectangle(element: DependencyObject,value: Rect)

   Sets the value of the System.Windows.Controls.ToolTipService.PlacementRectangle�attached 

    property for an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetPlacementTarget(element,value):
  """
  SetPlacementTarget(element: DependencyObject,value: UIElement)

   Sets the value of the System.Windows.Controls.ToolTipService.PlacementTarget�attached property 

    for an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetShowDuration(element,value):
  """
  SetShowDuration(element: DependencyObject,value: int)

   Sets the value of the System.Windows.Controls.ToolTipService.ShowDuration�attached property for 

    an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetShowOnDisabled(element,value):
  """
  SetShowOnDisabled(element: DependencyObject,value: bool)

   Sets the value of the System.Windows.Controls.ToolTipService.ShowOnDisabled�attached property 

    for an object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetToolTip(element,value):
  """
  SetToolTip(element: DependencyObject,value: object)

   Sets the value of the System.Windows.Controls.ToolTipService.ToolTip�attached�property for an 

    object.

  

  

   element: The object to which the attached property is written.

   value: The value to set.
  """
  pass
 @staticmethod
 def SetVerticalOffset(element,value):
  """
  SetVerticalOffset(element: DependencyObject,value: float)

   Sets the value of the System.Windows.Controls.ToolTipService.VerticalOffset�attached property 

    for an object.

  

  

   element: The object to which the attached property is written.

   value: The desired value.
  """
  pass
 BetweenShowDelayProperty=None
 HasDropShadowProperty=None
 HorizontalOffsetProperty=None
 InitialShowDelayProperty=None
 IsEnabledProperty=None
 IsOpenProperty=None
 PlacementProperty=None
 PlacementRectangleProperty=None
 PlacementTargetProperty=None
 ShowDurationProperty=None
 ShowOnDisabledProperty=None
 ToolTipClosingEvent=None
 ToolTipOpeningEvent=None
 ToolTipProperty=None
 VerticalOffsetProperty=None
 __all__=[
  'AddToolTipClosingHandler',
  'AddToolTipOpeningHandler',
  'BetweenShowDelayProperty',
  'GetBetweenShowDelay',
  'GetHasDropShadow',
  'GetHorizontalOffset',
  'GetInitialShowDelay',
  'GetIsEnabled',
  'GetIsOpen',
  'GetPlacement',
  'GetPlacementRectangle',
  'GetPlacementTarget',
  'GetShowDuration',
  'GetShowOnDisabled',
  'GetToolTip',
  'GetVerticalOffset',
  'HasDropShadowProperty',
  'HorizontalOffsetProperty',
  'InitialShowDelayProperty',
  'IsEnabledProperty',
  'IsOpenProperty',
  'PlacementProperty',
  'PlacementRectangleProperty',
  'PlacementTargetProperty',
  'RemoveToolTipClosingHandler',
  'RemoveToolTipOpeningHandler',
  'SetBetweenShowDelay',
  'SetHasDropShadow',
  'SetHorizontalOffset',
  'SetInitialShowDelay',
  'SetIsEnabled',
  'SetPlacement',
  'SetPlacementRectangle',
  'SetPlacementTarget',
  'SetShowDuration',
  'SetShowOnDisabled',
  'SetToolTip',
  'SetVerticalOffset',
  'ShowDurationProperty',
  'ShowOnDisabledProperty',
  'ToolTipClosingEvent',
  'ToolTipOpeningEvent',
  'ToolTipProperty',
  'VerticalOffsetProperty',
 ]




class RunWorkerCompletedEventArgs(AsyncCompletedEventArgs):
 """
 Provides data for the MethodNameCompleted event.

 

 RunWorkerCompletedEventArgs(result: object,error: Exception,cancelled: bool)
 """
 @staticmethod
 def __new__(self,result,error,cancelled):
  """ __new__(cls: type,result: object,error: Exception,cancelled: bool) """
  pass
 Result=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that represents the result of an asynchronous operation.



Get: Result(self: RunWorkerCompletedEventArgs) -> object



"""

 UserState=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that represents the user state.



Get: UserState(self: RunWorkerCompletedEventArgs) -> object



"""





"""Find, parse and cache extensions.

There are two types of extensions: UI Extensions (components.Extension) and
Library Extensions (components.LibraryExtension).

This module, finds the ui extensions installed and parses their directory for
tools or loads them from cache. It also finds the library extensions and adds
their directory address to the ui extensions so the python tools can use
the shared libraries.

To do its job correctly, this module needs to communicate with
pyrevit.userconfig to get a list of user extension folder and also
pyrevit.extensions.extpackages to check whether an extension is active or not.
"""

from pyrevit import MAIN_LIB_DIR, MISC_LIB_DIR
from pyrevit.coreutils.logger import get_logger
from pyrevit.userconfig import user_config


try:
    if user_config.bin_cache:
        from pyrevit.extensions.cacher_bin import is_cache_valid,\
            get_cached_extension, update_cache
    else:
        from pyrevit.extensions.cacher_asc import is_cache_valid,\
            get_cached_extension, update_cache
except AttributeError:
    user_config.bin_cache = True
    user_config.save_changes()
    from pyrevit.extensions.cacher_bin import is_cache_valid,\
        get_cached_extension, update_cache

#pylint: disable=C0413
from pyrevit.extensions.parser import parse_dir_for_ext_type, get_parsed_extension
from pyrevit.extensions.components import Extension, LibraryExtension

import pyrevit.extensions.extpackages as extpkgs


#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


def _update_extension_search_paths(ui_ext, lib_ext_list, pyrvt_paths):
    for lib_ext in lib_ext_list:
        ui_ext.add_module_path(lib_ext.directory)

    for pyrvt_path in pyrvt_paths:
        ui_ext.add_module_path(pyrvt_path)


def _is_extension_enabled(ext_info):
    try:
        ext_pkg = extpkgs.get_ext_package_by_name(ext_info.name)
        if ext_pkg:
            return ext_pkg.is_enabled and ext_pkg.user_has_access
        else:
            mlogger.debug('Extension package is not defined: %s', ext_info.name)
    except Exception as ext_check_err:
        mlogger.error('Error checking state for extension: %s | %s',
                      ext_info.name, ext_check_err)

    # Lets be nice and load the package if it is not defined
    return True


def _remove_disabled_extensions(ext_list):
    cleaned_ext_list = []
    for extension in ext_list:
        if _is_extension_enabled(extension):
            cleaned_ext_list.append(extension)
        else:
            mlogger.debug('Skipping disabled extension: %s', extension.name)

    return cleaned_ext_list


def _parse_or_cache(ext_info):
    # parse the extension if ui_extension does not have a valid cache
    if not is_cache_valid(ext_info):
        mlogger.debug('Cache is not valid for: %s', ext_info)

        # Either cache is not available, not valid, or cache load has failed.
        # parse directory for components and return fully loaded ui_extension
        mlogger.debug('Parsing for ui_extension...')
        ui_extension = get_parsed_extension(ext_info)

        # update cache with newly parsed ui_extension
        mlogger.debug('UI Extension successfuly parsed: %s', ui_extension.name)
        mlogger.debug('Updating cache for ui_extension: %s', ui_extension.name)
        update_cache(ui_extension)

    # otherwise load the cache
    else:
        mlogger.debug('Cache is valid for: %s', ext_info)
        # if cache is valid, load the cached ui_extension
        # cacher module takes the ui_extension object and
        # injects cache data into it.
        ui_extension = get_cached_extension(ext_info)
        mlogger.debug('UI Extension successfuly loaded from cache: %s',
                     ui_extension.name)

    return ui_extension


def get_thirdparty_extension_data():
    """Returns all installed and active UI and Library extensions (not parsed).

    Returns:
        (list): list of components.Extension or components.LibraryExtension
    """
    # FIXME: reorganzie this code to use one single method to collect
    # extension data for both lib and ui
    ext_data_list = []

    for root_dir in user_config.get_thirdparty_ext_root_dirs():
        ext_data_list.extend(
            [ui_ext for ui_ext in parse_dir_for_ext_type(root_dir,
                                                         Extension)])
        ext_data_list.extend(
            [lib_ext for lib_ext in parse_dir_for_ext_type(root_dir,
                                                           LibraryExtension)])

    return _remove_disabled_extensions(ext_data_list)


def get_installed_lib_extensions(root_dir):
    """Returns all the installed and active Library extensions (not parsed).

    Args:
        root_dir (str): Extensions directory address

    Returns:
        (list[LibraryExtension]): list of components.LibraryExtension objects
    """
    lib_ext_list = \
        [lib_ext for lib_ext in parse_dir_for_ext_type(root_dir,
                                                       LibraryExtension)]
    return _remove_disabled_extensions(lib_ext_list)


def get_installed_ui_extensions():
    """Returns all UI extensions (fully parsed) under the given directory.

    This will also process the Library extensions and will add
    their path to the syspath of the UI extensions.

    Returns:
        (list[Extension]): list of components.Extension objects
    """
    ui_ext_list = []
    lib_ext_list = []

    # get a list of all directories that could include extensions
    ext_search_dirs = user_config.get_ext_root_dirs()
    mlogger.debug('Extension Directories: %s', ext_search_dirs)

    # collect all library extensions. Their dir paths need to be added
    # to sys.path for all commands
    for root_dir in ext_search_dirs:
        lib_ext_list.extend(get_installed_lib_extensions(root_dir))
        # Get a list of all installed extensions in this directory
        # _parser.parse_dir_for_ext_type() returns a list of extensions
        # in given directory

    for root_dir in ext_search_dirs:
        for ext_info in parse_dir_for_ext_type(root_dir, Extension):
            # test if cache is valid for this ui_extension
            # it might seem unusual to create a ui_extension and then
            # re-load it from cache but minimum information about the
            # ui_extension needs to be passed to the cache module for proper
            # hash calculation and ui_extension recovery. at this point
            # `ui_extension` does not include any sub-components
            # (e.g, tabs, panels, etc) ui_extension object is very small and
            # its creation doesn't add much overhead.

            if _is_extension_enabled(ext_info):
                ui_extension = _parse_or_cache(ext_info)
                ui_ext_list.append(ui_extension)
            else:
                mlogger.debug('Skipping disabled ui extension: %s',
                             ext_info.name)

    # update extension master syspaths with standard pyrevit lib paths and
    # lib address of other lib extensions (to support extensions that provide
    # library only to be used by other extensions)
    # all other lib paths internal to the extension and tool bundles have
    # already been set inside the extension bundles and will take precedence
    # over paths added by this method (they're the first paths added to the
    # search paths list, and these paths will follow)
    for ui_extension in ui_ext_list:
        _update_extension_search_paths(
            ui_extension,
            lib_ext_list,
            [MAIN_LIB_DIR, MISC_LIB_DIR]
            )

    return ui_ext_list



class ICommandExecutor:
 """ Defines a method that executes a certain action on the type that implements this interface. """
 def Execute(self):
  """
  Execute(self: ICommandExecutor)

   Performs a task that is determined by the type that implements this method.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class DnsEndPoint(EndPoint):
 """
 Represents a network endpoint as a host name or a string representation of an IP address and a port number.

 

 DnsEndPoint(host: str,port: int)

 DnsEndPoint(host: str,port: int,addressFamily: AddressFamily)
 """
 def Equals(self,comparand):
  """
  Equals(self: DnsEndPoint,comparand: object) -> bool

  

   Compares two System.Net.DnsEndPoint objects.

  

   comparand: A System.Net.DnsEndPoint instance to compare to the current instance.

   Returns: true if the two System.Net.DnsEndPoint instances are equal; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: DnsEndPoint) -> int

  

   Returns a hash value for a System.Net.DnsEndPoint.

   Returns: An integer hash value for the System.Net.DnsEndPoint.
  """
  pass
 def ToString(self):
  """
  ToString(self: DnsEndPoint) -> str

  

   Returns the host name or string representation of the IP address and port number of the 

    System.Net.DnsEndPoint.

  

   Returns: A string containing the address family,host name or IP address string,and the port number of 

    the specified System.Net.DnsEndPoint.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,host,port,addressFamily=None):
  """
  __new__(cls: type,host: str,port: int)

  __new__(cls: type,host: str,port: int,addressFamily: AddressFamily)
  """
  pass
 def __ne__(self,*args):
  pass
 AddressFamily=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the Internet Protocol (IP) address family.



Get: AddressFamily(self: DnsEndPoint) -> AddressFamily



"""

 Host=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the host name or string representation of the Internet Protocol (IP) address of the host.



Get: Host(self: DnsEndPoint) -> str



"""

 Port=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the port number of the System.Net.DnsEndPoint.



Get: Port(self: DnsEndPoint) -> int



"""





# encoding: utf-8
# module System.Drawing.Configuration calls itself Configuration
# from System.Drawing,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b03f5f7f11d50a3a
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class SystemDrawingSection(ConfigurationSection):
 """ SystemDrawingSection() """
 BitmapSuffix=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: BitmapSuffix(self: SystemDrawingSection) -> str



Set: BitmapSuffix(self: SystemDrawingSection)=value

"""

 ElementProperty=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Configuration.ConfigurationElementProperty object that represents the System.Configuration.ConfigurationElement object itself.



"""

 EvaluationContext=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Configuration.ContextInformation object for the System.Configuration.ConfigurationElement object.



"""

 HasContext=property(lambda self: object(),lambda self,v: None,lambda self: None)

 Properties=property(lambda self: object(),lambda self,v: None,lambda self: None)






"""Helper functions for working with revit files."""
import os
import os.path as op
import re
import codecs

from pyrevit import coreutils
from pyrevit.coreutils.logger import get_logger
from pyrevit.labs import TargetApps


#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


def cleanup_backups(main_revitfile):
    """Remove all incremental saves of the given Revit file."""
    file_dir = op.dirname(main_revitfile)
    fname, fext = op.splitext(op.basename(main_revitfile))
    backup_fname = re.compile(r'{}\.\d\d\d\d\.{}'.format(fname,
                                                         fext.replace('.', '')))
    for f in os.listdir(file_dir):
        if backup_fname.findall(f):
            file_path = op.join(file_dir, f)
            try:
                os.remove(file_path)
            except Exception as osremove_err:
                mlogger.error('Error backup file cleanup on: %s | %s',
                              file_path, osremove_err)


def correct_text_encoding(filename):
    """Convert encoding of text file generated by Revit to UTF-8.

    Args:
        filename (str): file path
    """
    fcontent = None
    if coreutils.check_encoding_bom(filename, bom_bytes=codecs.BOM_UTF16):
        with codecs.open(filename, 'r', 'utf_16_le') as oldf:
            fcontent = oldf.readlines()
    elif coreutils.check_encoding_bom(filename, bom_bytes=codecs.BOM_UTF8):
        with codecs.open(filename, 'r', 'utf_8') as oldf:
            fcontent = oldf.readlines()

    with codecs.open(filename, 'w', 'utf-8') as newf:
        newf.writelines(fcontent)


def read_text(filepath):
    """Safely read text files with Revit encoding."""
    with codecs.open(filepath, 'r', 'utf_16_le') as text_file:
        return text_file.read()


def write_text(filepath, contents):
    """Safely write text files with Revit encoding."""
    # if 'utf_16_le' is specified, codecs will not write the BOM
    with codecs.open(filepath, 'w', 'utf_16') as text_file:
        text_file.write(contents)


def get_file_info(filepath):
    """Returns file information.

    Args:
        filepath (str): file path

    Returns:
        (RevitModelFile): file information
    """
    return TargetApps.Revit.RevitModelFile(filepath)



"""LibGit2Sharp wrapper module for pyRevit.

Documentation:
    https://github.com/libgit2/libgit2sharp/wiki
"""

import os.path as op
from collections import OrderedDict

from pyrevit import HOST_APP, PyRevitException
from pyrevit.compat import safe_strtype, IRONPY
from pyrevit import framework
from pyrevit.framework import clr
from pyrevit.framework import DateTime, DateTimeOffset
from pyrevit.coreutils.logger import get_logger

# pylint: disable=W0703,C0302
mlogger = get_logger(__name__)  # pylint: disable=C0103


GIT_LIB = "LibGit2Sharp"

LIBGIT_DLL = framework.get_dll_file(GIT_LIB)
mlogger.debug("Loading dll: %s", LIBGIT_DLL)

try:
    if IRONPY:
        clr.AddReferenceToFileAndPath(LIBGIT_DLL)
    else:
        clr.AddReference(LIBGIT_DLL)

    import LibGit2Sharp as libgit  # pylint: disable=import-error

except Exception as load_err:
    mlogger.error(
        "Can not load %s module. "
        "This module is necessary for getting pyRevit version "
        "and staying updated. | %s",
        GIT_LIB,
        load_err,
    )


class PyRevitGitAuthenticationError(PyRevitException):
    """Git authentication error."""

    pass


class RepoInfo(object):
    """Repo wrapper for passing around repository information.

    Attributes:
        directory (str): repo directory
        name (str): repo name
        head_name (str): head branch name
        last_commit_hash (str): hash of head commit
        repo (str): ``LibGit2Sharp.Repository`` object
        branch (str): current branch name
        username (str): credentials - username
        password (str): credentials - password
    """

    def __init__(self, repo):
        self.directory = repo.Info.WorkingDirectory
        self.name = op.basename(op.normpath(self.directory))
        self.head_name = repo.Head.FriendlyName
        self.last_commit_hash = repo.Head.Tip.Id.Sha
        self.repo = repo
        self.branch = repo.Head.FriendlyName
        self.username = self.password = None

    def __repr__(self):
        return "<type 'RepoInfo' head '{}' @ {}>".format(
            self.last_commit_hash, self.directory
        )


def _credentials_hndlr(username, password):
    userpass = libgit.UsernamePasswordCredentials()
    userpass.Username = username
    userpass.Password = password
    return userpass


def _get_credentials_hndlr(username, password):
    return libgit.Handlers.CredentialsHandler(
        lambda url, uname, types: _credentials_hndlr(username, password)
    )


def _make_pull_options(repo_info):
    mlogger.debug("Making pull options: %s", repo_info)
    pull_ops = libgit.PullOptions()
    pull_ops.FetchOptions = libgit.FetchOptions()
    if repo_info.username and repo_info.password:
        mlogger.debug(
            "Making Credentials handler. "
            "(Username and password are available but"
            "will not be logged for privacy purposes.)"
        )

        pull_ops.FetchOptions.CredentialsProvider = _get_credentials_hndlr(
            repo_info.username, repo_info.password
        )

    return pull_ops


def _make_fetch_options(repo_info):
    mlogger.debug("Making fetch options: %s", repo_info)
    fetch_ops = libgit.FetchOptions()
    if repo_info.username and repo_info.password:
        mlogger.debug(
            "Making Credentials handler. "
            "(Username and password are available but"
            "will not be logged for privacy purposes.)"
        )

        fetch_ops.CredentialsProvider = _get_credentials_hndlr(
            repo_info.username, repo_info.password
        )

    return fetch_ops


def _make_clone_options(username=None, password=None):
    mlogger.debug("Making clone options.")
    clone_ops = libgit.CloneOptions()

    if username and password:
        mlogger.debug("Making Credentials handler.")
        creds_handler = _get_credentials_hndlr(username, password)

        # Only set the CredentialsProvider if it's a valid property
        if hasattr(clone_ops, "CredentialsProvider"):
            clone_ops.CredentialsProvider = creds_handler
        elif hasattr(clone_ops, "FetchOptions") and hasattr(
            clone_ops.FetchOptions, "CredentialsProvider"
        ):
            clone_ops.FetchOptions.CredentialsProvider = creds_handler
        else:
            mlogger.warning(
                "CloneOptions does not support CredentialsProvider. "
                "Skipping credentials."
            )

    return clone_ops


def _make_pull_signature():
    mlogger.debug("Creating pull signature for username: %s",
                  HOST_APP.username)
    return libgit.Signature(
        HOST_APP.username, HOST_APP.username, DateTimeOffset(DateTime.Now)
    )


def _process_git_error(exception_err):
    exception_msg = safe_strtype(exception_err)
    if "401" in exception_msg:
        raise PyRevitGitAuthenticationError(exception_msg)
    else:
        raise PyRevitException(exception_msg)


def get_repo(repo_dir):
    """Return repo object for given git repo directory.

    Args:
        repo_dir (str): full path of git repo directory

    Returns:
        (RepoInfo): repo object
    """
    repo = libgit.Repository(repo_dir)
    return RepoInfo(repo)


def git_pull(repo_info):
    """Pull the current head of given repo.

    Args:
        repo_info (RepoInfo): target repo object

    Returns:
        (RepoInfo): repo object with updated head
    """
    repo = repo_info.repo
    try:
        libgit.Commands.Pull(
            repo, _make_pull_signature(), _make_pull_options(repo_info)
        )

        mlogger.debug("Successfully pulled repo: %s", repo_info.directory)
        head_msg = safe_strtype(repo.Head.Tip.Message).replace("\n", "")

        mlogger.debug("New head is: %s > %s", repo.Head.Tip.Id.Sha, head_msg)
        return RepoInfo(repo)

    except Exception as pull_err:
        mlogger.debug("Failed git pull: %s | %s",
                      repo_info.directory, pull_err)
        _process_git_error(pull_err)


def git_fetch(repo_info):
    """Fetch current branch of given repo.

    Args:
        repo_info (RepoInfo): target repo object

    Returns:
        (RepoInfo): repo object with updated head
    """
    repo = repo_info.repo
    try:
        libgit.Commands.Fetch(
            repo,
            repo.Head.TrackedBranch.RemoteName,
            [],
            _make_fetch_options(repo_info),
            "fetching pyrevit updates",
        )

        mlogger.debug("Successfully pulled repo: %s", repo_info.directory)
        head_msg = safe_strtype(repo.Head.Tip.Message).replace("\n", "")

        mlogger.debug("New head is: %s > %s", repo.Head.Tip.Id.Sha, head_msg)
        return RepoInfo(repo)

    except Exception as fetch_err:
        mlogger.debug("Failed git fetch: %s | %s",
                      repo_info.directory, fetch_err)
        _process_git_error(fetch_err)


def git_clone(repo_url, clone_dir, username=None, password=None):
    """Clone git repository to given location.

    Args:
        repo_url (str): repo .git url
        clone_dir (str): destination path
        username (str): credentials - username
        password (str): credentials - password
    """
    try:
        libgit.Repository.Clone(
            repo_url,
            clone_dir,
            _make_clone_options(username=username, password=password),
        )

        mlogger.debug("Completed git clone: %s @ %s", repo_url, clone_dir)

    except Exception as clone_err:
        mlogger.debug(
            "Error cloning repo: %s to %s | %s", repo_url, clone_dir, clone_err
        )
        _process_git_error(clone_err)


def compare_branch_heads(repo_info):
    """Compare local and remote branch heads and return ???

    Args:
        repo_info (RepoInfo): target repo object
    """
    # FIXME: need return type. possibly simplify
    repo = repo_info.repo
    repo_branches = repo.Branches

    mlogger.debug("Repo branches: %s", [b.FriendlyName for b in repo_branches])

    for branch in repo_branches:
        if branch.FriendlyName == repo_info.branch and not branch.IsRemote:
            try:
                if branch.TrackedBranch:
                    mlogger.debug(
                        "Comparing heads: %s of %s",
                        branch.CanonicalName,
                        branch.TrackedBranch.CanonicalName,
                    )

                    hist_div = repo.ObjectDatabase.CalculateHistoryDivergence(
                        branch.Tip, branch.TrackedBranch.Tip
                    )
                    return hist_div
            except Exception as compare_err:
                mlogger.error(
                    "Can not compare branch %s in repo: %s | %s",
                    branch,
                    repo,
                    safe_strtype(compare_err).replace("\n", ""),
                )
        else:
            mlogger.debug("Skipping remote branch: %s", branch.CanonicalName)


def get_all_new_commits(repo_info):
    """Fetch and return new commits ahead of current head.

    Args:
        repo_info (RepoInfo): target repo object

    Returns:
        (OrderedDict[str, str]): ordered dict of commit hash:message
    """
    repo = repo_info.repo
    current_commit = repo_info.last_commit_hash

    ref_commit = repo.Lookup(libgit.ObjectId(current_commit),
                             libgit.ObjectType.Commit)

    # Let's only consider the refs that lead to this commit...
    refs = repo.Refs.ReachableFrom([ref_commit])

    # ...and create a filter that will retrieve all the commits...
    commit_filter = libgit.CommitFilter()
    commit_filter.IncludeReachableFrom = refs
    commit_filter.ExcludeReachableFrom = ref_commit
    commit_filter.SortBy = libgit.CommitSortStrategies.Time

    commits = repo.Commits.QueryBy(commit_filter)
    commitsdict = OrderedDict()
    for commit in commits:
        if commit in repo.Head.Commits or commit in repo.Head.TrackedBranch.Commits:
            commitsdict[commit.Id.ToString()] = commit.MessageShort

    return commitsdict



class Container(object,IContainer,IDisposable):
 """
 Encapsulates zero or more components.

 

 Container()
 """
 def Add(self,component,name=None):
  """
  Add(self: Container,component: IComponent,name: str)

   Adds the specified System.ComponentModel.Component to the System.ComponentModel.Container and 

    assigns it a name.

  

  

   component: The component to add.

   name: The unique,case-insensitive name to assign to the component.-or- null,which leaves the 

    component unnamed.

  

  Add(self: Container,component: IComponent)

   Adds the specified System.ComponentModel.Component to the System.ComponentModel.Container. The 

    component is unnamed.

  

  

   component: The component to add.
  """
  pass
 def CreateSite(self,*args):
  """
  CreateSite(self: Container,component: IComponent,name: str) -> ISite

  

   Creates a site System.ComponentModel.ISite for the given System.ComponentModel.IComponent and 

    assigns the given name to the site.

  

  

   component: The System.ComponentModel.IComponent to create a site for.

   name: The name to assign to component,or null to skip the name assignment.

   Returns: The newly created site.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Container)

   Releases all resources used by the System.ComponentModel.Container.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Container,service: Type) -> object

  

   Gets the service object of the specified type,if it is available.

  

   service: The System.Type of the service to retrieve.

   Returns: An System.Object implementing the requested service,or null if the service cannot be resolved.
  """
  pass
 def Remove(self,component):
  """
  Remove(self: Container,component: IComponent)

   Removes a component from the System.ComponentModel.Container.

  

   component: The component to remove.
  """
  pass
 def RemoveWithoutUnsiting(self,*args):
  """
  RemoveWithoutUnsiting(self: Container,component: IComponent)

   Removes a component from the System.ComponentModel.Container without setting 

    System.ComponentModel.IComponent.Site to null.

  

  

   component: The component to remove.
  """
  pass
 def ValidateName(self,*args):
  """
  ValidateName(self: Container,component: IComponent,name: str)

   Determines whether the component name is unique for this container.

  

   component: The named component.

   name: The component name to validate.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+yx.__add__(y) <==> x+y """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Components=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets all the components in the System.ComponentModel.Container.



Get: Components(self: Container) -> ComponentCollection



"""


